/*
 * main.c
 *
 * Copyright (C) 2015 Alexander Andrejevic <theflash AT sdf DOT lonestar DOT org>
 * Copyright (C) 2015 Jason Self <j@jxself.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <time.h>
#include <ctype.h>

#include "config.h"

enum { ZVERSION = 11, ZORKID, ZSERIAL };

static struct option const long_options[] =
{
    { "help",     no_argument,       NULL, 'h' },
    { "version",  no_argument,       NULL, 'V' },
    { "zversion", required_argument, NULL, ZVERSION },
    { "zorkid",   required_argument, NULL, ZORKID   },
    { "serial",   required_argument, NULL, ZSERIAL  },
    { NULL, 0, NULL, 0 }
};

struct
{
    int  zversion;     /* 0 - 8   */
    int  zorkid;       /* 0 - 255 */
    char zserial[7];   /* YYMMDD  */
} Config;

void wrong_arg()
{
    fprintf(stderr, "Try `" PACKAGE_NAME " --help' for more information.\n");
    exit(1);
}

void print_version()
{
    printf(PACKAGE_STRING "\n"
           "License AGPLv3+: GNU AGPL version 3 or later\n"
           "<http://gnu.org/licenses/agpl.html>\n"
           "This is free software: you are free to change and redistribute it.\n"
           "There is NO WARRANTY, to the extent permitted by law.\n"
    );

    exit(0);
}

void print_usage(int failed)
{
    printf("Usage: " PACKAGE_NAME " [OPTION...] [FILES...]\n"
           "\n"
           "--version  Display program version and exit\n"
           "--help     Display this help\n"
           "\n"
           "--zversion (accepts numbers 1 - 8, defaults to 1 if not specified)\n"
           "--zorkid   (accepts digits, defaults to 0 if not specified)\n"
           "--serial   (six characters of ASCII, defaults to current date\n"
           "            in the form YYMMDD if not specified)\n"
    );

    exit(failed);
}

void fill_zserial(void)
{
    time_t t;
    struct tm *timeinfo;
    time(&t);
    timeinfo = localtime(&t);
    strftime(Config.zserial, sizeof(Config.zserial), "%y%m%d", timeinfo);
}

void fill_config(void)
{
    Config.zversion = 1;
    Config.zorkid   = 0;
    fill_zserial();
}

void parse_intarg(int *dest, const char name[], int min, int max, int defval)
{
    if (!optarg)
    {
        *dest = defval;
        return;
    }

    int n = atoi(optarg);
    if (n >= min && n <= max)
    {
        *dest = n;
        return;
    }

    fprintf(stderr, "Wrong %s value %s, must be integer between %d and %d\n",
            name, optarg, min, max);
    wrong_arg();
}

void parse_zserial(void)
{
    if (!optarg)
    {
        fill_zserial();
        return;
    }

    size_t n = strlen(optarg);
    if (n == sizeof(Config.zserial) - 1)
    {
        char *p = optarg;
        while (*p && isalnum(*p))
            p++;

        if (!*p) /* ..optarg contains alphanumeric only? */
        {
            strncpy(Config.zserial, optarg, sizeof(Config.zserial));
            return;
        }
    }

    fprintf(stderr, "Wrong zserial value %s, must be 6 ASCII characters\n", optarg);
    wrong_arg();
}

int main(int argc, char *argv[], char *envp[])
{
    fill_config();

    int opt = 0;
    while ((opt = getopt_long(argc, argv, "hV", long_options, NULL)) != -1)
    {
        switch (opt)
        {
            case 'h'     : print_usage(0);
            case 'V'     : print_version();
            case ZVERSION: parse_intarg(&Config.zversion, "zversion", 1, 8,   1); break;
            case ZORKID  : parse_intarg(&Config.zorkid,   "zorkid",   0, 255, 0); break;
            case ZSERIAL : parse_zserial();  break;
            default      : wrong_arg();
        }
    }

    // TODO: Everything :)

    printf("Config:\n"
           "- ZVersion: %d\n"
           "- ZorkID:   %d\n"
           "- ZSerial:  %s\n",
           Config.zversion, Config.zorkid, Config.zserial
    );

    return 0;
}
